
#ifndef ROAD_H_INCLUDED
#define ROAD_H_INCLUDED

// the number buffered road slices (must be >> viewDY), and also must be 2^n
#define ROAD_SLICES 1024

// road bits
#define ROAD_BIT_FINISH_LINE (1 << 0)
#define ROAD_BIT_ROADSIDE_AD (1 << 1)

class Road {

 public:

	// the road widths per scanline (24.8 FP)
	unsigned int *roadDX;

	// the on screen coordinates for the road scanline pieces
	int *roadStartX;
	int *roadEndX;

	// the road turn ringbuffer
	int *roadTurn;

	// road slice marker bits
	unsigned char *roadBits;

	// the road pointers
	int *roadPointers;

 public:

	Road(void);
	~Road(void);

	// precalculate the road
	void precalculateRoad(int viewDX, int viewDY);

	// reset the road / level
	void reset(void);

	// update the road ringbuffers
	int update(Car *carPlayer);

	// draw the road
	void draw(volatile unsigned short *vram, Car *carPlayer, SpriteManager *spriteManager, Viewport *viewport);

	// get the angle of the road under the specified car
	int getAngle(Car *car);

 private:

	// unpack the specified road segment into the road ringbuffers
	void unpack(int segment);
};

#endif
